@@ -48,7 +48,8 @@ shim_loader_type = NamedTuple(
 grub_loader_type = NamedTuple(
     'grub_loader_type', [
         ('filename', str),
-        ('binaryname', str)
+        ('binaryname', str),
+        ('targetname', str)
     ]
 )
 
@@ -96,6 +97,9 @@ CUSTOM_RUNTIME_CONFIG_FILE = None
 PLATFORM_MACHINE = platform.machine()
 EFI_FAT_IMAGE_SIZE = 20
 
+# optional package manager environment variables
+PACKAGE_MANAGER_ENV_VARS = '/.kiwi.package_manager.env'
+
 log = logging.getLogger('kiwi')
 
 
@@ -388,6 +392,14 @@ class Defaults:
         """
         return 'removed'
 
+    @staticmethod
+    def get_system_files_name():
+        """
+        Provides base file name to store system files
+        in a container build
+        """
+        return 'systemfiles'
+
     @staticmethod
     def get_exclude_list_for_removed_files_detection() -> List[str]:
         """
@@ -432,19 +444,30 @@ class Defaults:
             return yaml.safe_load(meta)
 
     @staticmethod
-    def get_exclude_list_from_custom_exclude_files(root_dir: str) -> List:
+    def _parse_exclude_file(root_dir: str, exclude_filename: str) -> List:
         """
-        Provides the list of folders that are excluded by the
-        optional metadata file image/exclude_files.yaml
+        Retrieves an exclusion list from the provided metadata file
 
-        :return: list of file and directory names
+        The file should contain a YAML dictionary with a top-level key
+        named 'exclude' and the list of exclusions as its value.
+
+        The list of exclusions may include:
+            * file paths
+            * folder paths
+            * glob patterns
+
+        Paths and patterns should be relative to the filesystem or
+        directory that they're being excluded from.
+
+        :return: list of paths and glob patterns
 
         :param string root_dir: image root directory
+        :param string exclude_filename: file exclusion YAML metadata file
 
         :rtype: list
         """
         exclude_file = os.sep.join(
-            [root_dir, 'image', 'exclude_files.yaml']
+            [root_dir, 'image', exclude_filename]
         )
         exclude_list = []
         if os.path.isfile(exclude_file):
@@ -462,6 +485,36 @@ class Defaults:
                     )
         return exclude_list
 
+    @staticmethod
+    def get_exclude_list_from_custom_exclude_files(root_dir: str) -> List:
+        """
+        Gets the list of excluded items for the root filesystem from
+        the optional metadata file image/exclude_files.yaml
+
+        :return: list of paths and glob patterns
+
+        :param string root_dir: image root directory
+
+        :rtype: list
+        """
+        return Defaults._parse_exclude_file(root_dir, 'exclude_files.yaml')
+
+    @staticmethod
+    def get_exclude_list_from_custom_exclude_files_for_efifatimage(root_dir: str) -> List:
+        """
+        Gets the list of excluded items for the ESP's EFI folder from
+        the optional metadata file image/exclude_files_efifatimage.yaml
+
+        Excluded items must be relative to the ESP's /EFI directory.
+
+        :return: list of paths and glob patterns
+
+        :param string root_dir: EFI root directory
+
+        :rtype: list
+        """
+        return Defaults._parse_exclude_file(root_dir, 'exclude_files_efifatimage.yaml')
+
     @staticmethod
     def get_exclude_list_for_non_physical_devices():
         """
@@ -718,9 +771,9 @@ class Defaults:
         return modules
 
     @staticmethod
-    def get_grub_bios_modules(multiboot=False):
+    def get_grub_platform_modules(multiboot=False):
         """
-        Provides list of grub bios modules
+        Provides list of platform specific grub modules
 
         :param bool multiboot: grub multiboot mode
 
@@ -728,15 +781,19 @@ class Defaults:
 
         :rtype: list
         """
-        modules = Defaults.get_grub_basic_modules(multiboot) + [
-            'part_gpt',
-            'part_msdos',
-            'biosdisk',
-            'vga',
-            'vbe',
-            'chain',
-            'boot'
-        ]
+        modules = Defaults.get_grub_basic_modules(multiboot)
+        if Defaults.is_ppc64_arch(Defaults.get_platform_name()):
+            return Defaults.get_grub_ofw_modules()
+        else:
+            modules += [
+                'part_gpt',
+                'part_msdos',
+                'biosdisk',
+                'vga',
+                'vbe',
+                'chain',
+                'boot'
+            ]
         return modules
 
     @staticmethod
@@ -772,7 +829,9 @@ class Defaults:
         return modules
 
     @staticmethod
-    def get_grub_path(root_path, filename, raise_on_error=True):
+    def get_grub_path(
+        root_path: str, filename: str, raise_on_error: bool = True
+    ) -> str:
         """
         Provides grub path to given search file
 
@@ -814,6 +873,7 @@ class Defaults:
             raise KiwiBootLoaderGrubDataError(
                 'grub path {0} not found in {1}'.format(filename, lookup_list)
             )
+        return ''
 
     @staticmethod
     def get_preparer():
@@ -838,7 +898,7 @@ class Defaults:
         return 'SUSE LINUX GmbH'
 
     @staticmethod
-    def get_shim_loader(root_path: str) -> Optional[shim_loader_type]:
+    def get_shim_loader(root_path: str) -> List[shim_loader_type]:
         """
         Provides shim loader file path
 
@@ -847,39 +907,73 @@ class Defaults:
 
         :param string root_path: image root path
 
-        :return: shim_loader_type | None
+        :return: list of shim_loader_type
 
-        :rtype: NamedTuple
+        :rtype: list
         """
-
+        result = []
         shim_pattern_type = namedtuple(
             'shim_pattern_type', ['pattern', 'binaryname']
         )
-
         shim_file_patterns = [
-            shim_pattern_type('/usr/lib/shim/shim*.efi.signed.latest', 'shimx64.efi'),
-            shim_pattern_type('/usr/lib/shim/shim*.efi.signed', 'shimx64.efi'),
-            shim_pattern_type('/usr/lib/grub/*-efi-signed', 'shimx64.efi'),
-            shim_pattern_type('/usr/share/efi/*/shim.efi', None),
-            shim_pattern_type('/usr/lib64/efi/shim.efi', None),
-            shim_pattern_type('/boot/efi/EFI/*/shim[a-z]*.efi', None),
-            shim_pattern_type('/boot/efi/EFI/*/shim.efi', None),
-            shim_pattern_type('/usr/lib/shim/shim*.efi', None)
+            shim_pattern_type(
+                '/usr/lib/shim/shim*.efi.signed.latest',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/lib/shim/shim*.efi.signed',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/lib/grub/*-efi-signed',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/share/efi/x86_64/shim.efi',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/share/efi/aarch64/shim.efi',
+                'bootaa64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/lib64/efi/shim.efi',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/boot/efi/EFI/*/shimx64.efi',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/boot/efi/EFI/*/shimia32.efi',
+                'bootia32.efi'
+            ),
+            shim_pattern_type(
+                '/boot/efi/EFI/*/shimaa64.efi',
+                'bootaa64.efi'
+            ),
+            shim_pattern_type(
+                '/boot/efi/EFI/*/shim.efi',
+                'bootx64.efi'
+            ),
+            shim_pattern_type(
+                '/usr/lib/shim/shim*.efi',
+                'bootx64.efi'
+            )
         ]
         for shim_file_pattern in shim_file_patterns:
-            for shim_file in sorted(glob.iglob(root_path + shim_file_pattern.pattern), key=len):
-                if not shim_file_pattern.binaryname:
-                    binaryname = os.path.basename(shim_file)
-                else:
-                    binaryname = shim_file_pattern.binaryname
-                return shim_loader_type(
-                    shim_file, binaryname
+            for shim_file in sorted(
+                glob.iglob(root_path + shim_file_pattern.pattern), key=len
+            ):
+                result.append(
+                    shim_loader_type(shim_file, shim_file_pattern.binaryname)
                 )
-
-        return None
+                # one match only expected, per pattern
+                break
+        return result
 
     @staticmethod
-    def get_mok_manager(root_path: str) -> Optional[str]:
+    def get_mok_manager(root_path: str) -> List[str]:
         """
         Provides Mok Manager file path
 
@@ -892,6 +986,7 @@ class Defaults:
 
         :rtype: str
         """
+        result = []
         mok_manager_file_patterns = [
             '/usr/share/efi/*/MokManager.efi',
             '/usr/lib64/efi/MokManager.efi',
@@ -900,8 +995,8 @@ class Defaults:
         ]
         for mok_manager_file_pattern in mok_manager_file_patterns:
             for mm_file in glob.iglob(root_path + mok_manager_file_pattern):
-                return mm_file
-        return None
+                result.append(mm_file)
+        return result
 
     @staticmethod
     def get_grub_efi_font_directory(root_path):
@@ -924,7 +1019,7 @@ class Defaults:
     @staticmethod
     def get_unsigned_grub_loader(
         root_path: str, target_type: str = 'disk'
-    ) -> Optional[str]:
+    ) -> List[grub_loader_type]:
         """
         Provides unsigned grub efi loader file path
 
@@ -933,35 +1028,115 @@ class Defaults:
 
         :param string root_path: image root path
 
-        :return: file path or None
+        :return: list of grub_loader_type
 
-        :rtype: str
+        :rtype: list
         """
-        unsigned_grub_file = None
+        result = []
+        grub_pattern_type = namedtuple(
+            'grub_pattern_type', ['pattern', 'binaryname', 'targetname']
+        )
         unsigned_grub_file_patterns = {
             'disk': [
-                '/usr/share/grub*/*-efi/grub.efi',
-                '/usr/lib/grub*/*-efi/grub.efi',
-                '/boot/efi/EFI/*/grubx64.efi',
-                '/boot/efi/EFI/*/grubaa64.efi'
+                grub_pattern_type(
+                    '/usr/share/grub*/x86_64-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/lib/grub*/x86_64-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubx64.efi',
+                    'grubx64.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubia32.efi',
+                    'grubia32.efi',
+                    'bootia32.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubaa64.efi',
+                    'grubaa64.efi',
+                    'bootaa64.efi'
+                )
             ],
             'iso': [
-                '/boot/efi/EFI/*/gcdx64.efi',
-                '/usr/share/grub*/*-efi/grub.efi',
-                '/usr/lib/grub*/*-efi/grub.efi',
-                '/boot/efi/EFI/*/grubx64.efi',
-                '/boot/efi/EFI/*/grubaa64.efi'
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/gcdx64.efi',
+                    'grubx64.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/share/grub*/x86_64-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/lib/grub*/x86_64-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubx64.efi',
+                    'grubx64.efi',
+                    'bootx64.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubia32.efi',
+                    'grubia32.efi',
+                    'bootia32.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubaa64.efi',
+                    'grubaa64.efi',
+                    'bootaa64.efi'
+                )
             ]
         }
         for unsigned_grub_file_pattern in unsigned_grub_file_patterns[target_type]:
             for unsigned_grub_file in glob.iglob(
-                root_path + unsigned_grub_file_pattern
+                root_path + unsigned_grub_file_pattern.pattern
             ):
-                return unsigned_grub_file
-        return None
+                result.append(
+                    grub_loader_type(
+                        unsigned_grub_file,
+                        unsigned_grub_file_pattern.binaryname,
+                        unsigned_grub_file_pattern.targetname
+                    )
+                )
+                # one match only expected, per pattern
+                break
+        return result
 
     @staticmethod
-    def get_grub_bios_core_loader(root_path):
+    def get_grub_chrp_loader(boot_path: str) -> str:
+        """
+        Lookup CHRP boot loader (ppc)
+
+        :param string boot_path: boot path
+
+        :return: file base name
+
+        :rtype: str
+        """
+        for chrp_loader in ['grub.elf', 'core.elf']:
+            for grub_chrp in glob.iglob(
+                os.sep.join(
+                    [boot_path, 'boot/grub*/powerpc-ieee1275', chrp_loader]
+                )
+            ):
+                log.info(f'Found CHRP loader at: {grub_chrp}')
+                return os.path.basename(grub_chrp)
+        raise KiwiBootLoaderGrubDataError(
+            f'CHRP loader not found in {boot_path}'
+        )
+
+    @staticmethod
+    def get_grub_platform_core_loader(root_path):
         """
         Provides grub bios image
 
@@ -976,10 +1151,12 @@ class Defaults:
         """
         bios_grub_core_patterns = [
             '/usr/share/grub*/{0}/{1}'.format(
-                Defaults.get_bios_module_directory_name(), Defaults.get_bios_image_name()
+                Defaults.get_grub_platform_module_directory_name(),
+                Defaults.get_grub_platform_image_name()
             ),
             '/usr/lib/grub*/{0}/{1}'.format(
-                Defaults.get_bios_module_directory_name(), Defaults.get_bios_image_name()
+                Defaults.get_grub_platform_module_directory_name(),
+                Defaults.get_grub_platform_image_name()
             )
         ]
         for bios_grub_core_pattern in bios_grub_core_patterns:
@@ -1013,7 +1190,7 @@ class Defaults:
     @staticmethod
     def get_signed_grub_loader(
         root_path: str, target_type: str = 'disk'
-    ) -> Optional[grub_loader_type]:
+    ) -> List[grub_loader_type]:
         """
         Provides shim signed grub loader file path
 
@@ -1022,70 +1199,117 @@ class Defaults:
 
         :param str root_path: image root path
 
-        :return: grub_loader_type | None
+        :return: list of grub_loader_type
 
-        :rtype: NamedTuple
+        :rtype: list
         """
+        result = []
         grub_pattern_type = namedtuple(
-            'grub_pattern_type', ['pattern', 'binaryname']
+            'grub_pattern_type', ['pattern', 'binaryname', 'targetname']
         )
         signed_grub_file_patterns = {
             'disk': [
                 grub_pattern_type(
-                    '/usr/share/efi/*/grub.efi', None
+                    '/usr/share/efi/*/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
-                    '/usr/lib64/efi/grub.efi', None
+                    '/usr/lib64/efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
-                    '/boot/efi/EFI/*/grub*.efi', None
+                    '/boot/efi/EFI/*/grubx64.efi',
+                    'grubx64.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
-                    '/usr/share/grub*/*-efi/grub.efi', None
+                    '/boot/efi/EFI/*/grubia32.efi',
+                    'grubia32.efi',
+                    'bootia32.efi'
+                ),
+                grub_pattern_type(
+                    '/boot/efi/EFI/*/grubaa64.efi',
+                    'grubaa64.efi',
+                    'bootaa64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/share/grub*/*-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
                     '/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed',
-                    'grubx64.efi'
+                    'grubx64.efi',
+                    'bootx64.efi'
                 )
             ],
             'iso': [
                 grub_pattern_type(
                     '/boot/efi/EFI/*/gcdx64.efi',
-                    'grubx64.efi'
+                    'grubx64.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
                     '/boot/efi/EFI/*/gcdaa64.efi',
+                    'grubaa64.efi',
+                    'bootaa64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/share/efi/x86_64/grub.efi',
+                    'grub.efi',
+                    'grubx64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/share/efi/aarch64/grub.efi',
+                    'grub.efi',
                     'grubaa64.efi'
                 ),
                 grub_pattern_type(
-                    '/usr/share/efi/*/grub.efi', None
+                    '/usr/lib64/efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
-                    '/usr/lib64/efi/grub.efi', None
+                    '/boot/efi/EFI/*/grubx64.efi',
+                    'grubx64.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
-                    '/boot/efi/EFI/*/grub*.efi', None
+                    '/boot/efi/EFI/*/grubia32.efi',
+                    'grubia32.efi',
+                    'bootia32.efi'
                 ),
                 grub_pattern_type(
-                    '/usr/share/grub*/*-efi/grub.efi', None
+                    '/boot/efi/EFI/*/grubaa64.efi',
+                    'grubaa64.efi',
+                    'bootaa64.efi'
+                ),
+                grub_pattern_type(
+                    '/usr/share/grub*/x86_64-efi/grub.efi',
+                    'grub.efi',
+                    'bootx64.efi'
                 ),
                 grub_pattern_type(
                     '/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed',
-                    'grubx64.efi'
+                    'grubx64.efi',
+                    'bootx64.efi'
                 )
-
             ]
         }
         for signed_grub in signed_grub_file_patterns[target_type]:
             for signed_grub_file in glob.iglob(root_path + signed_grub.pattern):
-                if not signed_grub.binaryname:
-                    binaryname = os.path.basename(signed_grub_file)
-                else:
-                    binaryname = signed_grub.binaryname
-                return grub_loader_type(
-                    signed_grub_file, binaryname
+                result.append(
+                    grub_loader_type(
+                        signed_grub_file,
+                        signed_grub.binaryname,
+                        signed_grub.targetname
+                    )
                 )
-        return None
+                # one match only expected, per pattern
+                break
+        return result
 
     @staticmethod
     def get_efi_vendor_directory(efi_path):
@@ -1299,6 +1523,7 @@ class Defaults:
             'armv7hl': ['efi', 'uefi'],
             'armv7l': ['efi', 'uefi'],
             'armv8l': ['efi', 'uefi'],
+            'loongarch64': ['efi', 'uefi'],
             'ppc': ['ofw'],
             'ppc64': ['ofw', 'opal'],
             'ppc64le': ['ofw', 'opal'],
@@ -1323,6 +1548,7 @@ class Defaults:
             'i586': 'bios',
             'i686': 'bios',
             'ix86': 'bios',
+            'loongarch64': 'efi',
             'ppc': 'ofw',
             'ppc64': 'ofw',
             'ppc64le': 'ofw',
@@ -1380,6 +1606,7 @@ class Defaults:
         """
         default_module_directory_names = {
             'x86_64': 'x86_64-efi',
+            'i386': 'i386-efi',
 
             # There is no dedicated xen architecture but there are
             # modules provided for xen. Thus we treat it as an
@@ -1393,21 +1620,25 @@ class Defaults:
             'armv6l': 'arm-efi',
             'armv7l': 'arm-efi',
             'armv8l': 'arm-efi',
+            'loongarch64': 'loongarch64-efi',
             'riscv64': 'riscv64-efi'
         }
         if arch in default_module_directory_names:
             return default_module_directory_names[arch]
 
     @staticmethod
-    def get_bios_module_directory_name():
+    def get_grub_platform_module_directory_name():
         """
-        Provides BIOS directory name which stores the pc binaries
+        Provides grub platform specific directory name which
+        stores the grub module binaries
 
         :return: directory name
 
         :rtype: str
         """
-        return 'powerpc-ieee1275' if Defaults.is_ppc64_arch(Defaults.get_platform_name()) else 'i386-pc'
+        return 'powerpc-ieee1275' if Defaults.is_ppc64_arch(
+            Defaults.get_platform_name()
+        ) else 'i386-pc'
 
     @staticmethod
     def get_efi_image_name(arch):
@@ -1422,6 +1653,7 @@ class Defaults:
         """
         default_efi_image_names = {
             'x86_64': 'bootx64.efi',
+            'i386': 'bootia32.efi',
             'aarch64': 'bootaa64.efi',
             'arm64': 'bootaa64.efi',
             'armv5el': 'bootarm.efi',
@@ -1429,21 +1661,24 @@ class Defaults:
             'armv6l': 'bootarm.efi',
             'armv7l': 'bootarm.efi',
             'armv8l': 'bootarm.efi',
+            'loongarch64': 'bootloongarch64.efi',
             'riscv64': 'bootriscv64.efi'
         }
         if arch in default_efi_image_names:
             return default_efi_image_names[arch]
 
     @staticmethod
-    def get_bios_image_name():
+    def get_grub_platform_image_name():
         """
-        Provides bios core boot binary name
+        Provides platform specific core boot binary name
 
         :return: name
 
         :rtype: str
         """
-        return 'grub.elf' if Defaults.is_ppc64_arch(Defaults.get_platform_name()) else 'core.img'
+        return 'grub.elf' if Defaults.is_ppc64_arch(
+            Defaults.get_platform_name()
+        ) else 'core.img'
 
     @staticmethod
     def get_default_boot_timeout_seconds():
@@ -1511,7 +1746,7 @@ class Defaults:
 
         :rtype: list
         """
-        return ['docker', 'oci', 'appx']
+        return ['docker', 'oci', 'appx', 'wsl']
 
     @staticmethod
     def get_filesystem_image_types():
@@ -2046,6 +2281,17 @@ class Defaults:
         """
         return '/boot/loader/entries'
 
+    @staticmethod
+    def get_apk_repo_config() -> str:
+        """
+        Repository file for apk
+
+        :return: file path name
+
+        :rtype: str
+        """
+        return '/etc/apk/repositories'
+
     def get(self, key):
         """
         Implements get method for profile elements
