@@ -1,9 +1,12 @@
 import logging
-from unittest.mock import patch
-
+from unittest.mock import (
+    patch, call
+)
 import sys
 from unittest.mock import MagicMock
-from pytest import fixture
+from pytest import (
+    fixture, raises
+)
 
 from .test_helper import argv_kiwi_tests
 
@@ -11,6 +14,8 @@ from kiwi.defaults import Defaults
 from kiwi.defaults import grub_loader_type
 from kiwi.defaults import shim_loader_type
 
+from kiwi.exceptions import KiwiBootLoaderGrubDataError
+
 
 class TestDefaults:
     @fixture(autouse=True)
@@ -86,12 +91,18 @@ class TestDefaults:
     @patch('kiwi.defaults.glob.iglob')
     def test_get_unsigned_grub_loader(self, mock_glob):
         mock_glob.return_value = ['/usr/share/grub2/x86_64-efi/grub.efi']
-        assert Defaults.get_unsigned_grub_loader('root') == \
-            mock_glob.return_value.pop()
-        mock_glob.assert_called_once_with('root/usr/share/grub*/*-efi/grub.efi')
+        assert mock_glob.return_value[0] in \
+            Defaults.get_unsigned_grub_loader('root')[0].filename
+        assert mock_glob.call_args_list == [
+            call('root/usr/share/grub*/x86_64-efi/grub.efi'),
+            call('root/usr/lib/grub*/x86_64-efi/grub.efi'),
+            call('root/boot/efi/EFI/*/grubx64.efi'),
+            call('root/boot/efi/EFI/*/grubia32.efi'),
+            call('root/boot/efi/EFI/*/grubaa64.efi')
+        ]
         mock_glob.reset_mock()
         mock_glob.return_value = []
-        assert Defaults.get_unsigned_grub_loader('root') is None
+        assert Defaults.get_unsigned_grub_loader('root') == []
 
     def test_is_x86_arch(self):
         assert Defaults.is_x86_arch('x86_64') is True
@@ -103,6 +114,23 @@ class TestDefaults:
         assert Defaults.get_vendor_grubenv('boot/efi') == \
             'boot/efi/EFI/fedora/grubenv'
 
+    def test_parse_exclude_file_is_valid(self):
+        assert Defaults._parse_exclude_file(
+            '../data/root-dir', 'exclude_files.yaml'
+        ) == [
+            'usr/bin/qemu-binfmt',
+            'usr/bin/qemu-x86_64-binfmt',
+            'usr/bin/qemu-x86_64'
+        ]
+
+    @patch('yaml.safe_load')
+    def test_parse_exclude_file_is_invalid(self, mock_yaml_safe_load):
+        mock_yaml_safe_load.return_value = {'invalid': 'artificial'}
+        with self._caplog.at_level(logging.WARNING):
+            assert Defaults._parse_exclude_file(
+                '../data/root-dir', 'exclude_files.yaml'
+            ) == []
+
     def test_get_exclude_list_from_custom_exclude_files(self):
         assert Defaults.get_exclude_list_from_custom_exclude_files(
             '../data/root-dir'
@@ -122,14 +150,32 @@ class TestDefaults:
                 '../data/root-dir'
             ) == []
 
+    def test_get_exclude_list_from_custom_exclude_files_for_efifatimage(self):
+        assert Defaults.get_exclude_list_from_custom_exclude_files_for_efifatimage(
+            '../data/root-dir'
+        ) == [
+            'BOOT/fbia32.efi',
+            'BOOT/fbx64.efi',
+        ]
+
+    @patch('yaml.safe_load')
+    def test_get_exclude_list_from_custom_exclude_files_for_efifatimage_is_invalid(
+        self, mock_yaml_safe_load
+    ):
+        mock_yaml_safe_load.return_value = {'invalid': 'artificial'}
+        with self._caplog.at_level(logging.WARNING):
+            assert Defaults.get_exclude_list_from_custom_exclude_files_for_efifatimage(
+                '../data/root-dir'
+            ) == []
+
     def test_get_exclude_list_for_root_data_sync(self):
         assert Defaults.get_exclude_list_for_root_data_sync() == [
-            'image', '.profile', '.kconfig',
+            'image', '.kconfig',
             'run/*', 'tmp/*',
             '.buildenv', 'var/cache/kiwi'
         ]
         assert Defaults.get_exclude_list_for_root_data_sync(no_tmpdirs=False) == [
-            'image', '.profile', '.kconfig',
+            'image', '.kconfig',
             '.buildenv', 'var/cache/kiwi'
         ]
 
@@ -139,8 +185,8 @@ class TestDefaults:
             return []
 
         def iglob_simple_match(pattern):
-            if '/boot/efi/EFI/*/grub*.efi' in pattern:
-                return ['root_path/boot/efi/EFI/BOOT/grub.efi']
+            if '/boot/efi/EFI/*/grubx64.efi' in pattern:
+                return ['root_path/boot/efi/EFI/BOOT/grubx64.efi']
             else:
                 return []
 
@@ -154,28 +200,40 @@ class TestDefaults:
                 return []
 
         mock_iglob.side_effect = iglob_no_matches
-        assert Defaults.get_signed_grub_loader('root_path') is None
+        assert Defaults.get_signed_grub_loader('root_path') == []
 
         mock_iglob.side_effect = iglob_simple_match
-        assert Defaults.get_signed_grub_loader('root_path') == grub_loader_type(
-            filename='root_path/boot/efi/EFI/BOOT/grub.efi',
-            binaryname='grub.efi'
-        )
+        assert Defaults.get_signed_grub_loader('root_path') == [
+            grub_loader_type(
+                filename='root_path/boot/efi/EFI/BOOT/grubx64.efi',
+                binaryname='grubx64.efi',
+                targetname='bootx64.efi'
+            )
+        ]
 
         mock_iglob.side_effect = iglob_custom_binary_match
-        assert Defaults.get_signed_grub_loader('root_path') == grub_loader_type(
-            filename='root_path'
-            '/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed',
-            binaryname='grubx64.efi'
-        )
+        assert Defaults.get_signed_grub_loader('root_path') == [
+            grub_loader_type(
+                filename='root_path'
+                '/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed',
+                binaryname='grubx64.efi',
+                targetname='bootx64.efi'
+            )
+        ]
 
     @patch('glob.iglob')
     def test_get_mok_manager(self, mock_iglob):
         mock_iglob.return_value = []
-        assert Defaults.get_mok_manager('root_path') is None
+        assert Defaults.get_mok_manager('root_path') == []
 
-        mock_iglob.return_value = ['some_glob_result']
-        assert Defaults.get_mok_manager('root_path') == 'some_glob_result'
+        def iglob_match(pattern):
+            if pattern == 'root_path/boot/efi/EFI/*/mm*.efi':
+                return ['some']
+            return []
+
+        mock_iglob.side_effect = iglob_match
+
+        assert Defaults.get_mok_manager('root_path') == ['some']
 
     @patch('glob.iglob')
     def test_get_shim_loader(self, mock_iglob):
@@ -186,29 +244,33 @@ class TestDefaults:
                 return []
 
         def iglob_custom_binary_match(pattern):
-            if '/usr/lib/shim/shimx64.efi.signed' in pattern:
+            if '/usr/lib/shim/shim*.efi.signed.latest' in pattern:
                 return [
                     'root_path'
-                    '/usr/lib/shim/shimx64.efi.signed'
+                    '/usr/lib/shim/shimx64.efi.signed.latest'
                 ]
             else:
                 return []
 
-        assert Defaults.get_shim_loader('root_path') is None
+        assert Defaults.get_shim_loader('root_path') == []
 
         mock_iglob.side_effect = iglob_simple_match
-        assert Defaults.get_shim_loader('root_path') == shim_loader_type(
-            filename='root_path'
-            '/usr/lib64/efi/shim.efi',
-            binaryname='shim.efi'
-        )
+        assert Defaults.get_shim_loader('root_path') == [
+            shim_loader_type(
+                filename='root_path'
+                '/usr/lib64/efi/shim.efi',
+                binaryname='bootx64.efi'
+            )
+        ]
 
         mock_iglob.side_effect = iglob_custom_binary_match
-        assert Defaults.get_shim_loader('root_path/usr/lib/shim/shimx64.efi.signed') == shim_loader_type(
-            filename='root_path'
-            '/usr/lib/shim/shimx64.efi.signed',
-            binaryname='shimx64.efi'
-        )
+        assert Defaults.get_shim_loader('root_path') == [
+            shim_loader_type(
+                filename='root_path'
+                '/usr/lib/shim/shimx64.efi.signed.latest',
+                binaryname='bootx64.efi'
+            )
+        ]
 
     @patch('os.path.exists')
     def test_get_snapper_config_template_file(self, mock_os_path_exists):
@@ -217,3 +279,30 @@ class TestDefaults:
         mock_os_path_exists.return_value = True
         assert Defaults.get_snapper_config_template_file('root') == \
             'root/etc/snapper/config-templates/default'
+
+    def test_get_min_volume_mbytes(self):
+        assert Defaults.get_min_volume_mbytes('btrfs') == 120
+        assert Defaults.get_min_volume_mbytes('xfs') == 300
+        assert Defaults.get_min_volume_mbytes('some') == 30
+
+    @patch('glob.iglob')
+    def test_get_grub_chrp_loader(self, mock_glob_iglob):
+        mock_glob_iglob.return_value = []
+        with raises(KiwiBootLoaderGrubDataError):
+            Defaults.get_grub_chrp_loader('some-boot')
+
+        mock_glob_iglob.reset_mock()
+        mock_glob_iglob.return_value = [
+            '/some-boot/boot/grub2/powerpc-ieee1275/grub.elf'
+        ]
+        assert Defaults.get_grub_chrp_loader('some-boot') == 'grub.elf'
+        mock_glob_iglob.assert_called_once_with(
+            'some-boot/boot/grub*/powerpc-ieee1275/grub.elf'
+        )
+
+    @patch('os.path.exists')
+    def test_get_grub_path(self, mock_os_path_exists):
+        mock_os_path_exists.return_value = False
+        assert Defaults.get_grub_path(
+            'root', 'some_file_to_search', False
+        ) == ''
